#include <stdlib.h>
#include <math.h>

#include "libaffin.h"

int
aff_single_bilinear_interpolation(int p[2][2], double x, double y)
{
	int k1, k2, k3;

	k1 = p[1][1] - p[1][0];
	k2 = p[0][1] - p[0][0] - k1;
	k3 = p[0][0] - p[1][0];

	return (int) (k1*x + k2*x*y + k3*y + p[1][0]);
}

int
aff_rotate_pic(int *dst, const int *src, int w, int h, int x0, int y0, double ang)
{
	double s, c;
	int i, j;

	if ((dst == NULL) || (src == NULL))
		return AFF_NULLPTR;

	s = sin(ang);
	c = cos(ang);

	for (i = 0; i < h; i++) {
		for (j = 0; j < w; j++) {
			double tx, ty;
			int p[2][2], pxy;
			int x, y, fx, fy, cx, cy; 

			/* simple rotation */
			x = j - x0;
			y = i - y0;

			tx =  c*x + s*y;
			ty = -s*x + c*y;

			tx += (double) x0;
			ty += (double) y0;

			/* prepare for interpolation */
			fx = (int) (floor(tx));
			fy = (int) (floor(ty));

			cx = fx + 1;
			cy = fy + 1;

			/* prevent the array border exceeding */
			p[0][0] = p[0][1] = p[1][0] = p[1][1] = 0;

			if ((w > fx) && (fx >= 0))
				if ((h > fy) && (fy >= 0))
					p[1][0] = src[fy*w+fx];

			if ((w > fx) && (fx >= 0))
				if ((h > cy) && (cy >= 0))
					p[1][1] = src[cy*w+fx];

			if ((w > cx) && (cx >= 0))
				if ((h > fy) && (fy >= 0))
					p[0][0] = src[fy*w+cx];

			if ((w > cx) && (cx >= 0))
				if ((h > cy) && (cy >= 0))
					p[0][1] = src[cy*w+cx];

			tx -= (double) fx;
			ty -= (double) fy;

			pxy = aff_single_bilinear_interpolation(p, tx, ty);

			dst[i*w+j] = pxy;
		}
	}

	return AFF_OK;
}

